@@ -1,14 +1,12 @@
+from flask_admin.base import AdminIndexView, expose
+from flask import render_template, redirect, request, url_for, json
 from flask_admin.contrib import sqla, fileadmin
-from flask.ext.admin.base import expose
-
-from shelf.admin.form.fields import LocalizedTextField, WysiwygTextField, LocalizedWysiwygTextField, RemoteFileField, PictureField
-from shelf.model.base import shelf_computed_models
 
 import os
 import os.path as op
 from operator import itemgetter
 
-from flask import request, url_for
+from flask import request, url_for, json
 from sqlalchemy.types import Text
 
 from sqlalchemy.sql.expression import desc
@@ -36,10 +34,62 @@ from werkzeug import secure_filename
 
 from flask_login import login_required
 
+from field import *
+
+import shutil
+
+import shelf.model
+db = shelf.model.db
+
+from model import LocalizedString, RemoteFile, LocalizedText, Picture
+
+def issubmodel(cls, name):
+    if not cls:
+        return False
+    if not name:
+        return False
+    if cls.__name__ == name:
+        return True
+    return False
+
+class DashboardView(AdminIndexView):
+    widgets = []
+
+    def add_widget(self, view):
+        if not self.widgets:
+            self.widgets = []
+        self.widgets.append(view)
+
+    @expose('/signin/', methods=("GET", "POST"))
+    def signin(self):
+    	if validate_login_on_submit(request.form):
+    		return redirect(request.args.get("next") or url_for("admin.index"))
+    	else:
+    		return render_template("shelf/login/login.html", form=LoginForm(request.form))
+
+    @expose('/signup/', methods=("GET", "POST"))
+    def signup(self):
+        if validate_register_on_submit(request.form):
+            return redirect(request.args.get("next") or url_for("admin.signin"))
+        else:
+    	   return render_template("shelf/login/signup.html", form=RegisterForm(request.form))
+
+    @expose('/recover/', methods=("GET", "POST"))
+    def recover(self):
+        if validate_recover_on_submit(request.form):
+            return redirect(request.args.get("next") or url_for("admin.signin"))
+        else:
+    	   return render_template("shelf/login/recover.html", form=RecoverForm(request.form))
+
+    @expose()
+    def index(self):
+        print self.widgets
+        return self.render(self._template, widgets=[w.render() for w in self.widgets])
+
 class ShelfModelView(sqla.ModelView):
-    list_template = "shelf-admin/model/list.html"
-    create_template = "shelf-admin/model/create.html"
-    edit_template = "shelf-admin/model/edit.html"
+    list_template = "shelf/model/list.html"
+    create_template = "shelf/model/create.html"
+    edit_template = "shelf/model/edit.html"
 
     def _order_by(self, query, joins, sort_field, sort_desc):
         """
@@ -54,54 +104,44 @@ class ShelfModelView(sqla.ModelView):
             :param sort_desc:
                 Ascending or descending
         """
+        if issubmodel(sort_field.mapper.class_, 'LocalizedString'):
+            table = sort_field.mapper.tables[0]
 
-        try:
-            if issubclass(sort_field.mapper.class_, shelf_computed_models()['LocalizedString']):
-                table = sort_field.mapper.tables[0]
+            query = query.outerjoin(str(sort_field).split('.')[1])
+            joins.add(table.name)
 
-                query = query.outerjoin(str(sort_field).split('.')[1])
-                joins.add(table.name)
+            if sort_desc:
+                query = query.order_by(desc(LocalizedString.value))
+            else:
+                query = query.order_by(LocalizedString.value)
 
-                if sort_desc:
-                    query = query.order_by(desc(shelf_computed_models()['LocalizedString'].value))
-                else:
-                    query = query.order_by(shelf_computed_models()['LocalizedString'].value)
+            return query, joins
 
-                return query, joins
-        except:
-            pass
+        if issubmodel(sort_field.mapper.class_, 'RemoteFile'):
+            table = sort_field.mapper.tables[0]
 
-        try:
-            if issubclass(sort_field.mapper.class_, shelf_computed_models()['RemoteFile']):
-                table = sort_field.mapper.tables[0]
+            query = query.outerjoin(str(sort_field).split('.')[1])
+            joins.add(table.name)
 
-                query = query.outerjoin(str(sort_field).split('.')[1])
-                joins.add(table.name)
+            if sort_desc:
+                query = query.order_by(desc(RemoteFile.path))
+            else:
+                query = query.order_by(RemoteFile.path)
 
-                if sort_desc:
-                    query = query.order_by(desc(shelf_computed_models()['RemoteFile'].path))
-                else:
-                    query = query.order_by(shelf_computed_models()['RemoteFile'].path)
+            return query, joins
 
-                return query, joins
-        except:
-            pass
+        if issubmodel(sort_field.mapper.class_, 'Picture'):
+            table = sort_field.mapper.tables[0]
 
-        try:
-            if issubclass(sort_field.mapper.class_, shelf_computed_models()['Picture']):
-                table = sort_field.mapper.tables[0]
+            query = query.outerjoin(str(sort_field).split('.')[1])
+            joins.add(table.name)
 
-                query = query.outerjoin(str(sort_field).split('.')[1])
-                joins.add(table.name)
+            if sort_desc:
+                query = query.order_by(desc(Picture.path))
+            else:
+                query = query.order_by(Picture.path)
 
-                if sort_desc:
-                    query = query.order_by(desc(shelf_computed_models()['Picture'].path))
-                else:
-                    query = query.order_by(shelf_computed_models()['Picture'].path)
-
-                return query, joins
-        except:
-            pass
+            return query, joins
 
         query, joins = super(ShelfModelView, self)._order_by(query, joins, sort_field, sort_desc)
         return query, joins
@@ -113,13 +153,12 @@ class ShelfModelView(sqla.ModelView):
         """
         columns = super(ShelfModelView, self).scaffold_sortable_columns()
 
+        
         for k, v in self.model.__dict__.items():
             if hasattr(v, "mapper"):
-                if 'LocalizedString' in shelf_computed_models() and issubclass(v.mapper.class_, shelf_computed_models()['LocalizedString']):
-                   columns[k] = v
-                if 'Picture' in shelf_computed_models() and issubclass(v.mapper.class_, shelf_computed_models()['Picture']):
-                   columns[k] = v
-                if 'RemoteFile' in shelf_computed_models() and issubclass(v.mapper.class_, shelf_computed_models()['RemoteFile']):
+                if issubmodel(v.mapper.class_, 'LocalizedString') or \
+                        issubmodel(v.mapper.class_, 'Picture') or \
+                        issubmodel(v.mapper.class_, 'RemoteFile'):
                    columns[k] = v
 
         return columns
@@ -141,12 +180,13 @@ class ShelfModelView(sqla.ModelView):
 
     @contextfunction
     def get_list_value(self, context, model, name):
-        if ('LocalizedString' in shelf_computed_models() and isinstance(getattr(model, name), shelf_computed_models()['LocalizedString'])) \
-              or ('LocalizedText' in shelf_computed_models() and isinstance(getattr(model, name), shelf_computed_models()['LocalizedText'])):
+        if issubmodel(getattr(model, name).__class__, 'LocalizedString') or \
+                issubmodel(getattr(model, name).__class__, 'LocalizedText'):
             return getattr(model, name).value
-        if ('RemoteFile' in shelf_computed_models() and isinstance(getattr(model, name), shelf_computed_models()['RemoteFile'])) \
-              or ('Picture' in shelf_computed_models() and isinstance(getattr(model, name), shelf_computed_models()['Picture'])):
+        elif issubmodel(getattr(model, name).__class__, 'Picture') or \
+                issubmodel(getattr(model, name).__class__, 'RemoteFile'):
             return getattr(model, name).path
+
         return super(ShelfModelView, self).get_list_value(context, model, name)
 
     @action('delete',
@@ -180,12 +220,10 @@ class ShelfModelView(sqla.ModelView):
             flash(gettext('Failed to delete models. %(error)s', error=str(ex)), 'error')
 
     @expose('/')
-    @login_required
     def index_view(self):
         return super(ShelfModelView, self).index_view()
 
     @expose('/new/', methods=('GET', 'POST'))
-    @login_required
     def create_view(self):
         """
             Create model view
@@ -215,7 +253,6 @@ class ShelfModelView(sqla.ModelView):
 
     
     @expose('/edit/', methods=('GET', 'POST'))
-    @login_required
     def edit_view(self):
         """
             Edit model view
@@ -261,90 +298,29 @@ class ShelfModelView(sqla.ModelView):
                 setattr(cls, k, WysiwygTextField())
 
             if hasattr(v, "mapper"):
-                if issubclass(v.mapper.class_, shelf_computed_models()['LocalizedString']) \
-                    and not issubclass(self.model, shelf_computed_models()['LocalizedString']):
-                        setattr(cls, k, LocalizedTextField())
-                elif 'LocalizedText' in shelf_computed_models() \
-                    and issubclass(v.mapper.class_, shelf_computed_models()['LocalizedText']) \
-                    and not issubclass(self.model, shelf_computed_models()['LocalizedText']):
-                        setattr(cls, k, LocalizedWysiwygTextField())
-                elif 'RemoteFile' in shelf_computed_models() \
-                    and issubclass(v.mapper.class_, shelf_computed_models()['RemoteFile']):
-                        setattr(cls, k, RemoteFileField())
-                elif 'Picture' in shelf_computed_models() \
-                    and issubclass(v.mapper.class_, shelf_computed_models()['Picture']):
-                        setattr(cls, k, PictureField())
+                if issubmodel(v.mapper.class_, 'LocalizedString') \
+                        and not issubmodel(self.model, 'LocalizedString'):
+                    setattr(cls, k, LocalizedTextField())
+                elif issubmodel(v.mapper.class_, 'LocalizedText') \
+                        and not issubmodel(self.model, 'LocalizedText'):
+                    setattr(cls, k, LocalizedWysiwygTextField())
+                elif issubmodel(v.mapper.class_, 'RemoteFile') \
+                        and not issubmodel(self.model, 'RemoteFile'):
+                    setattr(cls, k, RemoteFileField())
+                elif issubmodel(v.mapper.class_, 'Picture') \
+                        and not issubmodel(self.model, 'Picture'):
+                    setattr(cls, k, PictureField())
 
         return cls
 
-from flask.ext.admin.helpers import get_form_data
-
-class ShelfPageView(ShelfModelView):
-    can_create = False
-    can_delete = False
-
-    edit_template = "shelf-admin/page/edit.html"
-
-    pages_form = {}
-
-    def register_form(self, cls, form):
-        self.pages_form[cls] = form
-    
-    def edit_form(self, obj=None):
-        """
-            Instantiate model editing form and return it.
-
-            Override to implement custom behavior.
-        """
-        if not obj:
-            raise ValueError
-
-        if obj.__class__ in self.pages_form:
-            return self.pages_form[obj.__class__](get_form_data(), obj=obj)
-
-        converter = self.model_form_converter(self.session, self)
-        cls = contribform.get_form(obj.__class__, converter,
-                                   base_class=self.form_base_class,
-                                   only=self.form_columns,
-                                   exclude=self.form_excluded_columns,
-                                   field_args=self.form_args,
-                                   extra_fields=self.form_extra_fields)
-
-        if self.inline_models:
-            cls = self.scaffold_inline_form_models(form_class)
-
-        del cls.name
-
-        mapper = obj._sa_class_manager.mapper
-        properties = ((p.key, p) for p in mapper.iterate_properties)
-
-        for k, v in properties:
-            if hasattr(v, "type") and isinstance(v.type, Text):
-                setattr(cls, k, WysiwygTextField())
-
-            if hasattr(v, "mapper"):
-                if issubclass(v.mapper.class_, shelf_computed_models()['LocalizedString']) \
-                    and not issubclass(obj.__class__, shelf_computed_models()['LocalizedString']):
-                        setattr(cls, k, LocalizedTextField())
-                elif issubclass(v.mapper.class_, shelf_computed_models()['LocalizedText']) \
-                    and not issubclass(obj.__class__, shelf_computed_models()['LocalizedText']):
-                        setattr(cls, k, LocalizedWysiwygTextField())
-
-        return cls(get_form_data(), obj=obj)
-
-    def update_model(self, form, model):
-        return super(ShelfPageView, self).update_model(form, model)
-
-
-class ShelfFileAdmin(fileadmin.FileAdmin):
-    list_template = "shelf-admin/file/list.html"
-    iconic_template = "shelf-admin/file/iconic.html"
-    upload_template = "shelf-admin/file/upload.html"
-    modal_list_template = "shelf-admin/file/modal-list.html"
-    modal_iconic_template = "shelf-admin/file/modal-iconic.html"
+class Library(fileadmin.FileAdmin):
+    list_template = "shelf/library/list.html"
+    iconic_template = "shelf/library/iconic.html"
+    upload_template = "shelf/library/upload.html"
+    modal_list_template = "shelf/library/modal-list.html"
+    modal_iconic_template = "shelf/library/modal-iconic.html"
 
     @expose('/asyncupload', methods=("POST",))
-    @login_required
     def async_upload(self):
         mfile = request.form['file']
         mname = request.form['name']
@@ -356,13 +332,19 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
             f.write(decoded)
         return "True"
 
+
     @action('delete',
             lazy_gettext('Delete'),
             lazy_gettext('Are you sure you want to delete these files?'))
     def action_delete(self, items):
+        return_url = request.args.get('urls')
+
         if not self.can_delete:
             flash(gettext('File deletion is disabled.'), 'error')
-            return
+            if return_url:
+                return redirect(return_url) 
+            else:
+                return
 
         for path in items:
             base_path, full_path, path = self._normalize_path(path)
@@ -373,10 +355,58 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
                     flash(gettext('File "%(name)s" was successfully deleted.', name=path))
                 except Exception as ex:
                     flash(gettext('Failed to delete file: %(name)s', name=ex), 'error')
+                if return_url:
+                    return redirect(return_url) 
+                else:
+                    return
+
+
+    @expose('/delete/', methods=('POST',))
+    def delete(self):
+        """
+            Delete view method
+        """
+        path = request.form.get('path')
+
+        if not path:
+            return redirect(url_for('.index'))
+
+        # Get path and verify if it is valid
+        base_path, full_path, path = self._normalize_path(path)
+
+        return_url = request.args.get('url') or self._get_dir_url('.index', op.dirname(path))
+
+        if not self.can_delete:
+            flash(gettext('Deletion is disabled.'))
+            return redirect(return_url)
+
+        if not self.is_accessible_path(path):
+            flash(gettext(gettext('Permission denied.')))
+            return redirect(self._get_dir_url('.index'))
+
+        if op.isdir(full_path):
+            if not self.can_delete_dirs:
+                flash(gettext('Directory deletion is disabled.'))
+                return redirect(return_url)
+
+            try:
+                shutil.rmtree(full_path)
+                self.on_directory_delete(full_path, path)
+                flash(gettext('Directory "%s" was successfully deleted.' % path))
+            except Exception as ex:
+                flash(gettext('Failed to delete directory: %(error)s', error=ex), 'error')
+        else:
+            try:
+                os.remove(full_path)
+                self.on_file_delete(full_path, path)
+                flash(gettext('File "%(name)s" was successfully deleted.', name=path))
+            except Exception as ex:
+                flash(gettext('Failed to delete file: %(name)s', name=ex), 'error')
+
+        return redirect(return_url)
 
     @expose('/mkdir/', methods=('GET', 'POST'))
     @expose('/mkdir/<path:path>', methods=('GET', 'POST'))
-    @login_required
     def mkdir(self, path=None):
         """
             Directory creation view method
@@ -387,7 +417,7 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
         # Get path and verify if it is valid
         base_path, directory, path = self._normalize_path(path)
 
-        dir_url = self._get_dir_url('.index', path)
+        dir_url = request.args.get('url') or self._get_dir_url('.index', path)
 
         if not self.can_mkdir:
             flash(gettext('Directory creation is disabled.'), 'error')
@@ -398,7 +428,6 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
             return redirect(self._get_dir_url('.index'))
 
         form = NameForm(helpers.get_form_data())
-        print form, helpers.validate_form_on_submit(form), form.data
 
         if helpers.validate_form_on_submit(form):
             try:
@@ -411,9 +440,58 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
 
         return redirect(dir_url)
 
+
+    @expose('/rename/', methods=('GET', 'POST'))
+    def rename(self):
+        """
+            Rename view method
+        """
+        path = request.args.get('path')
+
+        if not path:
+            return redirect(url_for('.index'))
+
+        base_path, full_path, path = self._normalize_path(path)
+
+        return_url = request.args.get('url') or self._get_dir_url('.index', op.dirname(path))
+        
+        if not self.can_rename:
+            flash(gettext('Renaming is disabled.'))
+            return redirect(return_url)
+
+        if not self.is_accessible_path(path):
+            flash(gettext(gettext('Permission denied.')))
+            return redirect(self._get_dir_url('.index'))
+
+        if not op.exists(full_path):
+            flash(gettext('Path does not exist.'))
+            return redirect(return_url)
+
+        form = NameForm(helpers.get_form_data(), name=op.basename(path))
+        if helpers.validate_form_on_submit(form):
+            try:
+                dir_base = op.dirname(full_path)
+                filename = secure_filename(form.name.data)
+
+                os.rename(full_path, op.join(dir_base, filename))
+                self.on_rename(full_path, dir_base, filename)
+                flash(gettext('Successfully renamed "%(src)s" to "%(dst)s"',
+                      src=op.basename(path),
+                      dst=filename))
+            except Exception as ex:
+                flash(gettext('Failed to rename: %(error)s', error=ex), 'error')
+
+            return redirect(return_url)
+
+        return self.render(self.rename_template,
+                           form=form,
+                           path=op.dirname(path),
+                           name=op.basename(path),
+                           dir_url=return_url)
+
+
     @expose('/upload/', methods=('GET', 'POST'))
     @expose('/upload/<path:path>', methods=('GET', 'POST'))
-    @login_required
     def upload(self, path=None):
         """
             Upload view method
@@ -433,6 +511,7 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
             return redirect(self._get_dir_url('.index'))
 
         form = UploadForm(self)
+
         #print form, helpers.validate_form_on_submit(form), form.data
         if helpers.validate_form_on_submit(form):
             filename = op.join(directory,
@@ -445,15 +524,31 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
                 try:
                     self.save_file(filename, form.upload.data)
                     self.on_file_upload(directory, path, filename)
+                    flash('%s was correctly uploaded' % form.upload.data.filename)
                     return redirect(self._get_dir_url('.index', path))
                 except Exception as ex:
                     flash(gettext('Failed to save file: %(error)s', error=ex))
+        elif request.form and 'async' in request.form:
+            total_uploaded = 0
+            for tmp_filename in json.loads(request.form['async']):
+                filename = op.join(directory,
+                               secure_filename(form.upload.data.filename))
+                if op.exists(filename):
+                    total_uploaded = total_uploaded + 1
+
+            if total_uploaded == 0:
+                flash('Nothing was uploaded', 'error')
+            elif total_uploaded == 1:
+                flash('%s was correctly uploaded' % tmp_filename)
+                return redirect(self._get_dir_url('.index', path))
+            else:
+                flash('%d files were correctly uploaded' % total_uploaded)
+                return redirect(self._get_dir_url('.index', path))
 
         return self.render(self.upload_template, form=form, dir_path=path)
 
     @expose('/modal-icons/')
     @expose('/modal-icons/b/<path:path>')
-    @login_required
     def modal_iconic_index(self, path=None):
         """
             Index view method
@@ -525,7 +620,6 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
 
     @expose('/modal/')
     @expose('/modal/b/<path:path>')
-    @login_required
     def modal_index(self, path=None):
         """
             Index view method
@@ -597,7 +691,6 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
 
     @expose('/icons/')
     @expose('/icons/b/<path:path>')
-    @login_required
     def iconic_index(self, path=None):
         """
             Index view method
@@ -669,7 +762,6 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
 
     @expose('/')
     @expose('/b/<path:path>')
-    @login_required
     def index(self, path=None):
         """
             Index view method
@@ -737,4 +829,71 @@ class ShelfFileAdmin(fileadmin.FileAdmin):
                            items=items,
                            mimes=mimes,
                            actions=actions,
-                           actions_confirmation=actions_confirmation)
\ No newline at end of file
+                           actions_confirmation=actions_confirmation)
+
+
+from flask.ext.admin.helpers import get_form_data
+
+class ShelfPageView(ShelfModelView):
+    can_create = False
+    can_delete = False
+
+    edit_template = "shelf/model/edit-page.html"
+
+    pages_form = {}
+
+    column_list = ("name", "title", "description")
+
+    def register_form(self, cls, form):
+        self.pages_form[cls] = form
+    
+    def edit_form(self, obj=None):
+        """
+            Instantiate model editing form and return it.
+
+            Override to implement custom behavior.
+        """
+        if not obj:
+            raise ValueError
+
+        if obj.__class__ in self.pages_form:
+            return self.pages_form[obj.__class__](get_form_data(), obj=obj)
+
+        converter = self.model_form_converter(self.session, self)
+        cls = contribform.get_form(obj.__class__, converter,
+                                   base_class=self.form_base_class,
+                                   only=self.form_columns,
+                                   exclude=self.form_excluded_columns,
+                                   field_args=self.form_args,
+                                   extra_fields=self.form_extra_fields)
+
+        if self.inline_models:
+            cls = self.scaffold_inline_form_models(form_class)
+
+        del cls.name
+
+        mapper = obj._sa_class_manager.mapper
+        properties = ((p.key, p) for p in mapper.iterate_properties)
+
+        for k, v in properties:
+            if hasattr(v, "type") and isinstance(v.type, Text):
+                setattr(cls, k, WysiwygTextField())
+
+            if hasattr(v, "mapper"):
+                if issubmodel(v.mapper.class_, 'LocalizedString') \
+                        and not issubmodel(self.model, 'LocalizedString'):
+                    setattr(cls, k, LocalizedTextField())
+                elif issubmodel(v.mapper.class_, 'LocalizedText') \
+                        and not issubmodel(self.model, 'LocalizedText'):
+                    setattr(cls, k, LocalizedWysiwygTextField())
+                elif issubmodel(v.mapper.class_, 'RemoteFile') \
+                        and not issubmodel(self.model, 'RemoteFile'):
+                    setattr(cls, k, RemoteFileField())
+                elif issubmodel(v.mapper.class_, 'Picture') \
+                        and not issubmodel(self.model, 'Picture'):
+                    setattr(cls, k, PictureField())
+
+        return cls(get_form_data(), obj=obj)
+
+    def update_model(self, form, model):
+        return super(ShelfPageView, self).update_model(form, model)
